@@ -2,6 +2,8 @@
 #include "fileinfo_p.h"
 #include <gio/gio.h>
 
+#define METADATA_TRUST "metadata::trust"
+
 namespace Fm {
 
 const char defaultGFileInfoQueryAttribs[] = "standard::*,"
@@ -9,21 +11,29 @@ const char defaultGFileInfoQueryAttribs[] = "standard::*,"
                                             "time::*,"
                                             "access::*,"
                                             "id::filesystem,"
-                                            "metadata::emblems";
+                                            "metadata::emblems,"
+                                            METADATA_TRUST;
 
 FileInfo::FileInfo() {
     // FIXME: initialize numeric data members
 }
 
-FileInfo::FileInfo(const GFileInfoPtr& inf, const FilePath& parentDirPath) {
-    setFromGFileInfo(inf, parentDirPath);
+FileInfo::FileInfo(const GFileInfoPtr& inf, const FilePath& filePath, const FilePath& parentDirPath) {
+    setFromGFileInfo(inf, filePath, parentDirPath);
 }
 
 FileInfo::~FileInfo() {
 }
 
-void FileInfo::setFromGFileInfo(const GObjectPtr<GFileInfo>& inf, const FilePath& parentDirPath) {
-    dirPath_ = parentDirPath;
+void FileInfo::setFromGFileInfo(const GObjectPtr<GFileInfo>& inf, const FilePath& filePath, const FilePath& parentDirPath) {
+    inf_ = inf;
+    filePath_ = filePath;
+    if (filePath_ && filePath_.hasParent()) {
+        dirPath_ = filePath_.parent();
+    }
+    else {
+        dirPath_ = parentDirPath;
+    }
     const char* tmp, *uri;
     GIcon* gicon;
     GFileType type;
@@ -36,10 +46,9 @@ void FileInfo::setFromGFileInfo(const GObjectPtr<GFileInfo>& inf, const FilePath
     size_ = g_file_info_get_size(inf.get());
 
     tmp = g_file_info_get_content_type(inf.get());
-    if(!tmp) {
-        tmp = "application/octet-stream";
+    if(tmp) {
+        mimeType_ = MimeType::fromName(tmp);
     }
-    mimeType_ = MimeType::fromName(tmp);
 
     mode_ = g_file_info_get_attribute_uint32(inf.get(), G_FILE_ATTRIBUTE_UNIX_MODE);
 
@@ -119,7 +128,8 @@ void FileInfo::setFromGFileInfo(const GObjectPtr<GFileInfo>& inf, const FilePath
         isDeletable_ = true;
     }
 
-    isShortcut_ = false;
+    isShortcut_ = (type == G_FILE_TYPE_SHORTCUT);
+    isMountable_ = (type == G_FILE_TYPE_MOUNTABLE);
 
     /* special handling for symlinks */
     if(g_file_info_get_is_symlink(inf.get())) {
@@ -130,7 +140,6 @@ void FileInfo::setFromGFileInfo(const GObjectPtr<GFileInfo>& inf, const FilePath
 
     switch(type) {
     case G_FILE_TYPE_SHORTCUT:
-        isShortcut_ = true;
     /* Falls through. */
     case G_FILE_TYPE_MOUNTABLE:
         uri = g_file_info_get_attribute_string(inf.get(), G_FILE_ATTRIBUTE_STANDARD_TARGET_URI);
@@ -196,6 +205,10 @@ _file_is_symlink:
         }
     }
 
+    if(!mimeType_) {
+        mimeType_ = MimeType::fromName("application/octet-stream");
+    }
+
     /* if there is a custom folder icon, use it */
     if(isNative() && type == G_FILE_TYPE_DIRECTORY) {
         auto local_path = path().localPath();
@@ -377,6 +390,37 @@ bool FileInfo::isExecutableType() const {
     return mimeType_->canBeExecutable();
 }
 
+bool FileInfo::isTrustable() const {
+    if(isExecutableType()) {
+        /* to avoid GIO assertion warning: */
+        if(g_file_info_get_attribute_type(inf_.get(), METADATA_TRUST) == G_FILE_ATTRIBUTE_TYPE_STRING) {
+            if(const auto data = g_file_info_get_attribute_string(inf_.get(), METADATA_TRUST)) {
+                return (strcmp(data, "true") == 0);
+            }
+        }
+    }
+    return false;
+}
+
+void FileInfo::setTrustable(bool trust) const {
+    if(!isExecutableType()) {
+        return; // METADATA_TRUST is only for executables
+    }
+    GObjectPtr<GFileInfo> info {g_file_info_new()}; // used to set only this attribute
+    if(trust) {
+        g_file_info_set_attribute_string(info.get(), METADATA_TRUST, "true");
+        g_file_info_set_attribute_string(inf_.get(), METADATA_TRUST, "true");
+    }
+    else {
+        g_file_info_set_attribute(info.get(), METADATA_TRUST, G_FILE_ATTRIBUTE_TYPE_INVALID, nullptr);
+        g_file_info_set_attribute(inf_.get(), METADATA_TRUST, G_FILE_ATTRIBUTE_TYPE_INVALID, nullptr);
+    }
+    g_file_set_attributes_from_info(path().gfile().get(),
+                                    info.get(),
+                                    G_FILE_QUERY_INFO_NONE,
+                                    nullptr, nullptr);
+}
+
 
 bool FileInfoList::isSameType() const {
     if(!empty()) {
